% by Seungmoon Song
% Feb. 2014


function animInit(namestr, ViewWin)
      
global  SegObjects SegShape
    
  figure( ...
     'Tag',                          namestr, ...
     'Name',                         namestr, ...
     'NumberTitle',                    'off', ...
     'BackingStore',                   'off', ...
     'MenuBar',                       'default', ...
     'Color',                        [1 1 1], ...
     'Position',             [50 50 600 600], ...
     'Renderer',                    'OpenGL');

 
axis([ViewWin(1,1) ViewWin(1,2) ViewWin(2,1) ViewWin(2,2) ViewWin(3,1) ViewWin(3,2)]);
view(60, 25)
%view([1, 0, 0])
% axis off;
axis equal;
hold on;

% change some properties
set(gca, 'DrawMode',   'fast', ...
       'Color',     [1 1 1], ...
       'XColor',    [0 0 0], ...
       'YColor',    [0 0 0]);
   
lighting gouraud
camlight

SegCol      = [0.8 0.8 0.7];
[SegObjects SegShape] = CreateSegObjects(SegCol);

end


function [SegObjects, SegShape] = CreateSegObjects(SegCol)
 
    n_seg = 29;
    mechDir = '../0_meshes/';
    nameSeg = {'pelvis', 'ltorso', 'mtorso', 'utorso', 'head', ...
        'l_uglut', 'l_lglut', 'l_uleg', 'l_lleg', 'l_talus', 'l_foot', ...
        'r_uglut', 'r_lglut', 'r_uleg', 'r_lleg', 'r_talus', 'r_foot', ...
        'l_clav', 'l_scap', 'l_uarm', 'l_larm', 'l_farm', 'l_hand', ...
        'r_clav', 'r_scap', 'r_uarm', 'r_larm', 'r_farm', 'r_hand'};
            
    for i = 1:n_seg
        segFile = char(strcat(mechDir, nameSeg(i), '.stl'));
        [V, F, n, C, stltitle] = stlread(segFile);

        segPatch = patch('faces', F, 'vertices' ,V);
        set(segPatch, 'Visible', 'off');

        SegShape(i).x = get(segPatch, 'XData');
        SegShape(i).y = get(segPatch, 'YData');
        SegShape(i).z = get(segPatch, 'ZData');

        SegObjects(i) = surf(SegShape(i).x, SegShape(i).y, SegShape(i).z, 'FaceColor', SegCol); 
        
    end    
    
    set(SegObjects, 'Visible', 'on', 'EdgeColor', 'none', ...
                     'BackFaceLighting', 'unlit') 

          
                 
end

       
function [v, f, n, c, stltitle] = stlread(filename, verbose)
    % This function reads an STL file in binary format into vertex and face
    % matrices v and f.
    %
    % USAGE: [v, f, n, c, stltitle] = stlread(filename, verbose);
    %
    % verbose is an optional logical argument for displaying some loading
    %   information (default is false).
    %
    % v contains the vertices for all triangles [3*n x 3].
    % f contains the vertex lists defining each triangle face [n x 3].
    % n contains the normals for each triangle face [n x 3].
    % c is optional and contains color rgb data in 5 bits [n x 3].
    % stltitle contains the title of the specified stl file [1 x 80].
    %
    % To see plot the 3D surface use:
    %   patch('Faces',f,'Vertices',v,'FaceVertexCData',c);
    % or
    %   plot3(v(:,1),v(:,2),v(:,3),'.');
    %
    % Duplicate vertices can be removed using:
    %   [v, f]=patchslim(v, f);
    %
    % For more information see:
    %  http://www.esmonde-white.com/home/diversions/matlab-program-for-loading-stl-files
    %
    % Based on code originally written by:
    %    Doron Harlev
    % and combined with some code by:
    %    Eric C. Johnson, 11-Dec-2008
    %    Copyright 1999-2008 The MathWorks, Inc.
    % 
    % Re-written and optimized by Francis Esmonde-White, May 2010.

    use_color=(nargout>=4);

    fid=fopen(filename, 'r'); %Open the file, assumes STL Binary format.
    if fid == -1 
        error('File could not be opened, check name or path.')
    end

    if ~exist('verbose','var')
        verbose = false;
    end

    ftitle=fread(fid,80,'uchar=>schar'); % Read file title
    numFaces=fread(fid,1,'int32'); % Read number of Faces

    T = fread(fid,inf,'uint8=>uint8'); % read the remaining values
    fclose(fid);

    stltitle = char(ftitle');

    if verbose
        fprintf('\nTitle: %s\n', stltitle);
        fprintf('Number of Faces: %d\n', numFaces);
        disp('Please wait...');
    end

    % Each facet is 50 bytes
    %  - Three single precision values specifying the face normal vector
    %  - Three single precision values specifying the first vertex (XYZ)
    %  - Three single precision values specifying the second vertex (XYZ)
    %  - Three single precision values specifying the third vertex (XYZ)
    %  - Two color bytes (possibly zeroed)

    % 3 dimensions x 4 bytes x 4 vertices = 48 bytes for triangle vertices
    % 2 bytes = color (if color is specified)

    trilist = 1:48;

    ind = reshape(repmat(50*(0:(numFaces-1)),[48,1]),[1,48*numFaces])+repmat(trilist,[1,numFaces]);
    Tri = reshape(typecast(T(ind),'single'),[3,4,numFaces]);

    n=squeeze(Tri(:,1,:))';
    n=double(n);

    v=Tri(:,2:4,:);
    v = reshape(v,[3,3*numFaces]);
    v = double(v)';

    f = reshape(1:3*numFaces,[3,numFaces])';

    if use_color
        c0 = typecast(T(49:50),'uint16');
        if (bitget(c0(1),16)==1)
            trilist = 49:50;
            ind = reshape(repmat(50*(0:(numFaces-1)),[2,1]),[1,2*numFaces])+repmat(trilist,[1,numFaces]);
            c0 = reshape(typecast(T(ind),'uint16'),[1,numFaces]);

            r=bitshift(bitand(2^16-1, c0),-10);
            g=bitshift(bitand(2^11-1, c0),-5);
            b=bitand(2^6-1, c0);
            c=[r; g; b]';
        else
            c = zeros(numFaces,3);
        end
    end

    if verbose
        disp('Done!');
    end
end

